%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This m-file includes Matlab code for the ionosphere data experiments
% reported in article "Compact Modeling of Data Using Independent
% Variable Group Analysis" by Esa Alhoniemi, Antti Honkela, Krista
% Lagus, Jeremias Sepp, Paul Wagner, and Harri Valpola
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% create input data
% 
% The file ionosphere.data can be obtained from the UCI Machine
% Learning Repository
% http://www.ics.uci.edu/~mlearn/MLRepository.html

Data       = load('ionosphere.data', ',');
Data(:, 2) = []; % remove constant variable #2 from the
                 % dataset (contains no information for the
                 % classification)

Data_features = Data(:,1:33); % feature vectors
Classes       = Data(:, 34);  % class information for the feature vectors

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% kNN run using all data (k is varied between 1 and 29)
%

Accuracy = zeros(15, 1);

for k = 1:2:29
  Accuracy((k+1)/2) = 1 - bs(Data_features, Classes, 1000, 51, k);
end

Results_knn = Accuracy;

save fs_none.mat Results_knn

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IVGA

% batch IVGA run (1000 separate IVGA runs)


addpath /share/ivga/releases/ivga-1.14.5/program/ % change your own IVGA path here

result       = ivga(ivga_import_data(Data, ...
                                     'types', 'rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrn'), ...
                    'runs', 1000 ); 

% compute the variables which are most frequently in the same group
% with the class variable (these are stored in descending order in
% variable Inds)

Dep_matrix   = ivga_show(result, 'mode', 'var_pairs', 'nodisplay'); 
[Dummy Inds] = sort(Dep_matrix{1}(1:33,34), 'descend');

% k-NN runs using the features selected by IVGA
%
% We try different values for k and different number of variables (i
% first variables which are most frequently grouped in the same group
% by IVGA).

Variables_ivga  = [];
Results_ivga    = zeros(15, 1);

for i = 1:33
  Inds_part = Inds(1:i);
  Accuracy  = zeros(15, 1);

  for k = 1:2:29
    Accuracy((k+1)/2) = 1 - bs(Data_features(:, Inds_part), Classes, 1000, 51, k);
  end

  if (max(Accuracy) > max(Results_ivga))
    Variables_ivga = Inds_part;
    Results_ivga   = Accuracy;
  end

  fprintf('Best so far (%d/33): %.4f (%d variables)\n', i, max(Results_ivga), length(Variables_ivga));
end

save fs_ivga.mat Variables_ivga Results_ivga

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Mann-Whitney test (aka Wilcoxon rank sum test)

% compute rank sum for each variable (requires statistics toolbox)
% and sort the obtained p-values in ascending order

R = zeros(33,1);
for i = 1:33
  R(i) = ranksum(Data_features(find(Classes==1), i), Data_features(find(Classes==2), i));
end
[Dummy Inds] = sort(R);

% k-NN runs using the features selected by M-W test
%
% We try different values for k and different number of variables (i
% first variables with the lowest p-value).

Variables_mw = [];
Results_mw   = zeros(15, 1);

for i = 1:33
  Inds_part = Inds(1:i);
  Accuracy  = zeros(15, 1);

  for k = 1:2:29
    Accuracy((k+1)/2) = 1 - bs(Data_features(:, Inds_part), Classes, 1000, 51, k);
  end

  if (max(Accuracy) > max(Results_mw))
    Variables_mw = Inds_part;
    Results_mw   = Accuracy;
  end

  fprintf('Best so far (%d/33): %.4f (%d variables)\n', i, max(Results_mw), length(Variables_mw));
end

save fs_mw.mat Variables_mw Results_mw

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% sequential forward selection run
%

% first, we select the variables by using different values for k the
% maximum run time of the sffs is fixed to two hours (7200 s) unless
% the iteration has ended up in an infinite loop before that

Variables_sffs = [];
best           = 1;       

for k = 3:2:29
  [Best_group smallest_error] = sffs(Data_features, Classes, 100, 51, k, 7200);
  if (smallest_error < best)
    best           = smallest_error;
    Variables_sffs = Best_group;
  end

  save fs_sffs.mat Variables_sffs

end

% for the selected set of features, we re-estimate the accuracy of
% the classification br repeating the run 1000 times for each k
% (instead of 100 above)

Results_sffs = zeros(15, 1);
for k = 1:2:29
  Results_sffs((k+1)/2) = 1 - bs(Data_features(:, Variables_sffs), Classes, 1000, 51, k)
end

save fs_sffs.mat Variables_sffs Results_sffs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
